<?php
namespace AkTest\ServiceManager\AbstractFactory;

use Ak\ServiceManager\AbstractFactory\PluginManagerAbstractFactory;
use AkTest\ServiceManager\AbstractFactory\Mock\Plugin;
use AkTest\ServiceManager\AbstractFactory\Mock\PluginManager;
use AkTest\ServiceManager\AbstractFactory\Mock\PluginManagerInvalidConfigKey;
use AkTest\ServiceManager\AbstractFactory\Mock\PluginManagerInvalidInterface;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\InvalidServiceException;
use PHPUnit\Framework\TestCase;
use Laminas\ServiceManager\Factory\InvokableFactory;
use Laminas\ServiceManager\ServiceManager;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use stdClass;

/**
 * Class PluginManagerAbstractFactoryTest
 *
 * @package AkTest\ServiceManager\AbstractFactory
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/service-manager
 */
final class PluginManagerAbstractFactoryTest extends TestCase
{
    private function getMockContainer(array $config = []): ContainerInterface
    {
        $container = new ServiceManager($config['dependencies'] ?? []);
        $container->setService('config', $config);

        return $container;
    }

    public function testCanFigureOutIfItCanCreateAManager()
    {
        $container = $this->getMockContainer();
        $factory = new PluginManagerAbstractFactory();

        $this->assertFalse($factory->canCreate($container, PluginManagerInvalidInterface::class));
        $this->assertFalse($factory->canCreate($container, PluginManagerInvalidConfigKey::class));
        $this->assertTrue($factory->canCreate($container, PluginManager::class));
    }

    /**
     * @return void
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testWillThrowExceptionIfTryingToCreateANonPluginManager()
    {
        $container = $this->getMockContainer();
        $factory = new PluginManagerAbstractFactory();

        $this->expectException(InvalidServiceException::class);
        $factory($container, stdClass::class);
    }

    /**
     * @return void
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanCreateAConfiguredPluginManager()
    {
        $container = $this->getMockContainer([
            'test_plugins' => [
                'aliases' => [
                    'plugin' => Plugin::class,
                ],
                'factories' => [
                    Plugin::class => InvokableFactory::class,
                ],
            ],
        ]);

        $factory = new PluginManagerAbstractFactory();
        $manager = $factory($container, PluginManager::class);

        $this->assertTrue($manager->has('plugin'));
        $this->assertInstanceOf(Plugin::class, $manager->get('plugin'));
    }

    /**
     * @return void
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanBeUsedAsAbstractFactory()
    {
        $container = $this->getMockContainer([
            'dependencies' => [
                'abstract_factories' => [
                    PluginManagerAbstractFactory::class,
                ],
            ],
        ]);

        $this->assertTrue($container->has(PluginManager::class));
        $this->assertInstanceOf(PluginManager::class, $container->get(PluginManager::class));
    }

    /**
     * @return void
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanBeUsedAsRegularFactory()
    {
        $container = $this->getMockContainer([
            'dependencies' => [
                'factories' => [
                    PluginManager::class => PluginManagerAbstractFactory::class,
                ],
            ],
        ]);

        $this->assertTrue($container->has(PluginManager::class));
        $this->assertInstanceOf(PluginManager::class, $container->get(PluginManager::class));
    }
}
